import Resource from "./resource";

export abstract class Provider {
  constructor(
    public hash: string,
    public hashFormat: string,
  ) {}

  abstract get downloadUrl(): Resource;
}

export class UrlProvider extends Provider {
  constructor(
    public hash: string,
    public hashFormat: string,
    private url: string,
  ) {
    super(hash, hashFormat);
  }

  get downloadUrl(): Resource {
    return new Resource(this.url);
  }
}

export class ModrinthProvider extends UrlProvider {
  constructor(
    hash: string,
    hashFormat: string,
    url: string,
    public modId: string,
    public versionId: string,
  ) {
    super(hash, hashFormat, url);
  }
}

export class GitHubProvider extends UrlProvider {
  constructor(
    hash: string,
    hashFormat: string,
    url: string,
    public branch: string,
    public regex: string,
    public slug: string,
    public tag: string,
  ) {
    super(hash, hashFormat, url);
  }
}

export class CurseForgeProvider extends Provider {
  constructor(
    hash: string,
    hashFormat: string,
    public mode: string,
    public fileId: number,
    public projectId: number,
  ) {
    super(hash, hashFormat);
  }

  get downloadUrl(): Resource {
    return new Resource(
      `https://www.curseforge.com/api/v1/mods/${this.projectId}/files/${this.fileId}/download`,
    );
  }
}
